/*
 * fontch - read nroff output, swallow and mark font changes
 */

#include <stdio.h>
#include <ctype.h>

#define	STREQ(a, b)	(*(a) == *(b) && strcmp((a), (b)) == 0)

/* imports */
extern int optind;
extern char *optarg;
extern FILE *efopen();

/* exports */
char *progname;
int debug;

/* private */
static struct osseq {
	char c1, c2;
	char seq[3];
} osseq[] = {
	'=', '<', "lh",	/* for awf */
	'=', '>', "rh",	/* for awf */
	'+', 'o', "bu",
	'|', '-', "dg",
	'c', '/', "ct",
	'r', 'O', "rg",
	'c', 'O', "co",
	'j', 'f', "sc",
	'<', 'a', "*a",
	',', 'B', "*b",
	'<', 'y', "*g",
	'<', 'o', "*d",
	'-', 'c', "*e",
	',', 'L', "*z",
	',', 'n', "*y",
	'-', '0', "*h",
	',', 'i', "*i",
	'\\', '>', "*|",
	',', 'u', "*m",
	',', 'v', "*n",
	',', '3', "*c",
	'-', 'n', "*p",
	'-', 'o', "*s",
	'~', 't', "*t",
	'/', 'o', "*f",
	'|', 'u', "*q",
	'u', 'w', "*w",
	'-', 'O', "*H",
	'|', 'O', "*F",
	'|', 'U', "*Q",
	'_', '>', ">=",
	'_', '<', "<=",
	'_', '=', "==",
	'~', '=', "~=",
	'=', '/', "!=",
	'|', '^', "ua",
	'|', 'v', "da",
	'-', ':', "di",
	'+', '_', "+-",
	'~', 'V', "gr",
	'/', 'O', "es",
	'-', 'C', "mo",
	'|', '=', "dd",
	'O', 'o', "bs",
	'O', '=', "L1",
	/* could add endless cases here */
	0, 0, NULL
};
static int spaceign = 0;

/*
 * main - parse arguments and handle options
 */
main(argc, argv)
int argc;
char *argv[];
{
	int c, errflg = 0;

	progname = argv[0];
	while ((c = getopt(argc, argv, "ds")) != EOF)
		switch (c) {
		case 'd':
			++debug;
			break;
		case 's':
			++spaceign;	/* ignore font changes due to spaces */
			break;
		default:
			errflg++;
			break;
		}
	if (errflg) {
		(void) fprintf(stderr, "usage: %s [-ds] [file]...\n", progname);
		exit(2);
	}

	if (optind >= argc)
		process(stdin, "stdin");
	else
		for (; optind < argc; optind++)
			if (STREQ(argv[optind], "-"))
				process(stdin, "-");
			else {
				FILE *in = efopen(argv[optind], "r");

				process(in, argv[optind]);
				(void) fclose(in);
			}
	exit(0);
}

/*
 * process - process input file
 */
/* ARGSUSED inname */
process(in, inname)
register FILE *in;
char *inname;
{
	enum font { roman, bold, italic };
	register int lastc, c = EOF;
	register enum font thisfont;
	register int prevc = ' ';
	static enum font lastfont = roman;

	while ((lastc = (c != EOF? c: getc(in))) != EOF) {
		thisfont = roman;
		/* overstrikes lastc? */
		while ((c = getc(in)) == '\b' && (c = getc(in)) != EOF)
			if ((thisfont == italic || lastc == '_') && c != '_') {
				if (thisfont == bold)
					(void) fprintf(stderr,
			"%s: change from bold to italic within a character!\n",
						progname);
				thisfont = italic;
				lastc = c;
			} else if (lastc == c) {
				if (thisfont == italic)
					(void) fprintf(stderr,
			"%s: change from italic to bold within a character!\n",
						progname);
				thisfont = bold;
			} else if (nrosseq(lastc, c)) {
				c = EOF, lastc = EOF;	/* no further output */
				break;
			} else
				(void) fprintf(stderr,
					"%s: %c overstruck by %c; ugh!\n",
					progname, lastc, c);	/* can't happen */
		if (thisfont != lastfont && (!spaceign || lastc != ' ')) {
			(void) printf("\\f%c", (thisfont == roman? 'R':
				(thisfont == italic? 'I': 'B')));
			lastfont = thisfont;
		}
		if (lastc == '-' && (!isascii(prevc) || !isalnum(prevc)))
			(void) putchar('\\');	/* make dash long */
		if (lastc != EOF)
			(void) putchar(lastc);
		if (lastc == '\\')
			(void) putchar('e');	/* generate backslash */
#ifdef notdef					/* too slow */
		if (c != EOF)
			(void) ungetc(c, in);
#endif
		prevc = lastc;
	}
}

int
nrosseq(c1, c2)			/* (c1,c2) is an nroff overstrike sequence? */
register int c1, c2;
{
	register struct osseq *ossp;

	for (ossp = osseq; ossp->c1 != '\0'; ossp++)
		if (c1 == ossp->c1 && c2 == ossp->c2 ||
		    c1 == ossp->c2 && c2 == ossp->c1) {	/* backwards? */
			(void) printf("\\(%s", ossp->seq);
			return 1;
		}
	return 0;	
}
